@@ -1,14 +1,21 @@
-from typing import Dict, List, Tuple, Union
+from datetime import datetime, timedelta
+from typing import Dict, List, Optional, Tuple, Union
 
 import isodate
 import timely_beliefs as tb
+import timely_beliefs.utils as tb_utils
 from sqlalchemy.orm import Query
 
 from flexmeasures.data.config import db
+from flexmeasures.data.models.legacy_migration_utils import (
+    copy_old_sensor_attributes,
+    get_old_model_type,
+)
 from flexmeasures.data.models.user import User
-from flexmeasures.data.models.time_series import Sensor, TimedValue
+from flexmeasures.data.models.time_series import Sensor, TimedValue, TimedBelief
 from flexmeasures.data.models.generic_assets import (
     create_generic_asset,
+    GenericAsset,
     GenericAssetType,
 )
 from flexmeasures.utils.entity_address_utils import build_entity_address
@@ -16,7 +23,11 @@ from flexmeasures.utils.flexmeasures_inflection import humanize, pluralize
 
 
 class AssetType(db.Model):
-    """Describing asset types for our purposes"""
+    """
+    Describing asset types for our purposes
+
+    This model is now considered legacy. See GenericAssetType.
+    """
 
     name = db.Column(db.String(80), primary_key=True)
     # The name we want to see (don't unnecessarily capitalize, so it can be used in a sentence)
@@ -32,10 +43,14 @@ class AssetType(db.Model):
     yearly_seasonality = db.Column(db.Boolean(), nullable=False, default=False)
 
     def __init__(self, **kwargs):
-        generic_asset_type = GenericAssetType(
-            name=kwargs["name"], description=kwargs.get("hover_label", None)
-        )
-        db.session.add(generic_asset_type)
+        generic_asset_type = GenericAssetType.query.filter_by(
+            name=kwargs["name"]
+        ).one_or_none()
+        if not generic_asset_type:
+            generic_asset_type = GenericAssetType(
+                name=kwargs["name"], description=kwargs.get("hover_label", None)
+            )
+            db.session.add(generic_asset_type)
         super(AssetType, self).__init__(**kwargs)
         self.name = self.name.replace(" ", "_").lower()
         if "display_name" not in kwargs:
@@ -77,7 +92,11 @@ class AssetType(db.Model):
 
 
 class Asset(db.Model, tb.SensorDBMixin):
-    """Each asset is an energy- consuming or producing hardware. """
+    """
+    Each asset is an energy- consuming or producing hardware.
+
+    This model is now considered legacy. See GenericAsset and Sensor.
+    """
 
     id = db.Column(
         db.Integer, db.ForeignKey("sensor.id"), primary_key=True, autoincrement=True
@@ -109,25 +128,73 @@ class Asset(db.Model, tb.SensorDBMixin):
 
     def __init__(self, **kwargs):
 
+        if "unit" not in kwargs:
+            kwargs["unit"] = "MW"  # current default
+        super(Asset, self).__init__(**kwargs)
+
         # Create a new Sensor with unique id across assets, markets and weather sensors
-        # Also keep track of ownership.
+        # Also keep track of ownership by creating a GenericAsset and assigning the new Sensor to it.
         if "id" not in kwargs:
-            generic_assets_arg = kwargs.copy()
+
+            asset_type = get_old_model_type(
+                kwargs, AssetType, "asset_type_name", "asset_type"
+            )
+
+            # Set up generic asset
+            generic_asset_kwargs = {
+                **kwargs,
+                **copy_old_sensor_attributes(
+                    self,
+                    old_sensor_type_attributes=[
+                        "can_curtail",
+                        "can_shift",
+                    ],
+                    old_sensor_attributes=[
+                        "display_name",
+                        "min_soc_in_mwh",
+                        "max_soc_in_mwh",
+                        "soc_in_mwh",
+                        "soc_datetime",
+                        "soc_udi_event_id",
+                    ],
+                    old_sensor_type=asset_type,
+                ),
+            }
+
             if "owner_id" in kwargs:
                 owner = User.query.get(kwargs["owner_id"])
                 if owner:
-                    generic_assets_arg.update(account_id=owner.account_id)
-            new_generic_asset = create_generic_asset("asset", **generic_assets_arg)
-            new_sensor = Sensor(name=kwargs["name"], generic_asset=new_generic_asset)
+                    generic_asset_kwargs.update(account_id=owner.account_id)
+            new_generic_asset = create_generic_asset("asset", **generic_asset_kwargs)
+
+            # Set up sensor
+            new_sensor = Sensor(
+                name=kwargs["name"],
+                generic_asset=new_generic_asset,
+                **copy_old_sensor_attributes(
+                    self,
+                    old_sensor_type_attributes=[
+                        "is_consumer",
+                        "is_producer",
+                        "daily_seasonality",
+                        "weekly_seasonality",
+                        "yearly_seasonality",
+                        "weather_correlations",
+                    ],
+                    old_sensor_attributes=[
+                        "display_name",
+                        "capacity_in_mw",
+                        "market_id",
+                    ],
+                    old_sensor_type=asset_type,
+                ),
+            )
             db.session.add(new_sensor)
             db.session.flush()  # generates the pkey for new_sensor
             sensor_id = new_sensor.id
         else:
             # The UI may initialize Asset objects from API form data with a known id
             sensor_id = kwargs["id"]
-        if "unit" not in kwargs:
-            kwargs["unit"] = "MW"  # current default
-        super(Asset, self).__init__(**kwargs)
         self.id = sensor_id
         if self.unit != "MW":
             raise Exception("FlexMeasures only supports MW as unit for now.")
@@ -135,6 +202,15 @@ class Asset(db.Model, tb.SensorDBMixin):
         if "display_name" not in kwargs:
             self.display_name = humanize(self.name)
 
+        # Copy over additional columns from (newly created) Asset to (newly created) Sensor
+        if "id" not in kwargs:
+            db.session.add(self)
+            db.session.flush()  # make sure to generate each column for the old sensor
+            new_sensor.unit = self.unit
+            new_sensor.event_resolution = self.event_resolution
+            new_sensor.knowledge_horizon_fnc = self.knowledge_horizon_fnc
+            new_sensor.knowledge_horizon_par = self.knowledge_horizon_par
+
     asset_type = db.relationship("AssetType", backref=db.backref("assets", lazy=True))
     owner = db.relationship(
         "User",
@@ -144,6 +220,37 @@ class Asset(db.Model, tb.SensorDBMixin):
     )
     market = db.relationship("Market", backref=db.backref("assets", lazy=True))
 
+    def latest_state(self, event_ends_before: Optional[datetime] = None) -> "Power":
+        """Search the most recent event for this sensor, optionally before some datetime."""
+        # todo: replace with Sensor.latest_state
+        power_query = (
+            Power.query.filter(Power.sensor_id == self.id)
+            .filter(Power.horizon <= timedelta(hours=0))
+            .order_by(Power.datetime.desc())
+        )
+        if event_ends_before is not None:
+            power_query = power_query.filter(
+                Power.datetime + self.event_resolution <= event_ends_before
+            )
+        return power_query.first()
+
+    @property
+    def corresponding_sensor(self) -> Sensor:
+        return db.session.query(Sensor).get(self.id)
+
+    @property
+    def generic_asset(self) -> GenericAsset:
+        return db.session.query(GenericAsset).get(self.corresponding_sensor.id)
+
+    def get_attribute(self, attribute: str):
+        """Looks for the attribute on the corresponding Sensor.
+
+        This should be used by all code to read these attributes,
+        over accessing them directly on this class,
+        as this table is in the process to be replaced by the Sensor table.
+        """
+        return self.corresponding_sensor.get_attribute(attribute)
+
     @property
     def power_unit(self) -> float:
         """Return the 'unit' property of the generic asset, just with a more insightful name."""
@@ -216,18 +323,18 @@ class Power(TimedValue, db.Model):
     """
     All measurements of power data are stored in one slim table.
     Negative values indicate consumption.
-    TODO: datetime objects take up most of the space (12 bytes each)). One way out is to normalise them out to a table.
-    TODO: If there are more than one measurement per asset per time step possible, we can expand rather easily.
+
+    This model is now considered legacy. See TimedBelief.
     """
 
-    asset_id = db.Column(
+    sensor_id = db.Column(
         db.Integer(),
-        db.ForeignKey("asset.id", ondelete="CASCADE"),
+        db.ForeignKey("sensor.id", ondelete="CASCADE"),
         primary_key=True,
         index=True,
     )
-    asset = db.relationship(
-        "Asset",
+    sensor = db.relationship(
+        "Sensor",
         backref=db.backref(
             "measurements",
             lazy=True,
@@ -242,23 +349,54 @@ class Power(TimedValue, db.Model):
         **kwargs,
     ) -> Query:
         """Construct the database query."""
-        return super().make_query(asset_class=Asset, **kwargs)
+        return super().make_query(**kwargs)
 
     def to_dict(self):
         return {
             "datetime": isodate.datetime_isoformat(self.datetime),
-            "asset_id": self.asset_id,
+            "sensor_id": self.sensor_id,
             "value": self.value,
             "horizon": self.horizon,
         }
 
-    def __init__(self, **kwargs):
+    def __init__(self, use_legacy_kwargs: bool = True, **kwargs):
+        # todo: deprecate the 'asset_id' argument in favor of 'sensor_id' (announced v0.8.0)
+        if "asset_id" in kwargs and "sensor_id" not in kwargs:
+            kwargs["sensor_id"] = tb_utils.replace_deprecated_argument(
+                "asset_id",
+                kwargs["asset_id"],
+                "sensor_id",
+                None,
+            )
+            kwargs.pop("asset_id", None)
+
+        # todo: deprecate the 'Power' class in favor of 'TimedBelief' (announced v0.8.0)
+        if use_legacy_kwargs is False:
+            # Create corresponding TimedBelief
+            belief = TimedBelief(**kwargs)
+            db.session.add(belief)
+
+            # Convert key names for legacy model
+            kwargs["value"] = kwargs.pop("event_value")
+            kwargs["datetime"] = kwargs.pop("event_start")
+            kwargs["horizon"] = kwargs.pop("belief_horizon")
+            kwargs["sensor_id"] = kwargs.pop("sensor").id
+            kwargs["data_source_id"] = kwargs.pop("source").id
+
+        else:
+            import warnings
+
+            warnings.warn(
+                f"The {self.__class__} class is deprecated. Switch to using the TimedBelief class to suppress this warning.",
+                FutureWarning,
+            )
+
         super(Power, self).__init__(**kwargs)
 
     def __repr__(self):
-        return "<Power %.5f on Asset %s at %s by DataSource %s, horizon %s>" % (
+        return "<Power %.5f on Sensor %s at %s by DataSource %s, horizon %s>" % (
             self.value,
-            self.asset_id,
+            self.sensor_id,
             self.datetime,
             self.data_source_id,
             self.horizon,
