@@ -19,31 +19,23 @@ def logout(client):
 
 def mock_asset_response(
     asset_id: int = 1,
-    owner_id: int = 3,
-    market_id: int = 1,
+    account_id: int = 1,
     as_list: bool = True,
     multiple: bool = False,
 ) -> dict:
     asset = dict(
         id=asset_id,
         name="TestAsset",
-        display_name="New Test Asset",
-        asset_type_name="wind",
-        market_id=int(market_id),
-        owner_id=int(owner_id),
-        capacity_in_mw=100,
+        generic_asset_type_id=1,
+        account_id=int(account_id),
         latitude=70.4,
         longitude=30.9,
-        min_soc_in_mwh=0,
-        max_soc_in_mwh=0,
-        soc_in_mwh=0,
-        event_resolution=22,  # "PT15M",
     )
     if as_list:
         asset_list = [asset]
         if multiple:
             asset2 = copy.deepcopy(asset)
-            asset2["capacity_in_mw"] = 200
+            asset2["name"] = "TestAsset2"
             asset_list.append(asset2)
         return asset_list
     return asset
@@ -64,6 +56,7 @@ def mock_user_response(
         active=active,
         password="secret",
         flexmeasures_roles=[1],
+        last_login_at="2021-05-14T20:00:00+02:00",
     )
     if as_list:
         user_list = [user]
@@ -79,5 +72,5 @@ def mock_user_response(
 
 def mock_api_data_as_form_input(api_data: dict) -> dict:
     form_input = copy.deepcopy(api_data)
-    form_input["owner"] = api_data["owner_id"]
+    form_input["account"] = api_data["account_id"]
     return form_input
