@@ -86,6 +86,7 @@ class Config(object):
         str, List[Union[str, Tuple[str, List[str]]]]
     ] = "FlexMeasures"
     FLEXMEASURES_MODE: str = ""
+    FLEXMEASURES_ALLOW_DATA_OVERWRITE: bool = False
     FLEXMEASURES_TIMEZONE: str = "Asia/Seoul"
     FLEXMEASURES_HIDE_NAN_IN_UI: bool = False
     FLEXMEASURES_PUBLIC_DEMO_CREDENTIALS: Optional[Tuple] = None
@@ -94,20 +95,25 @@ class Config(object):
     # This setting contains the domain on which FlexMeasures runs
     # and the first month when the domain was under the current owner's administration
     FLEXMEASURES_HOSTS_AND_AUTH_START: dict = {"flexmeasures.io": "2021-01"}
-    FLEXMEASURES_PLUGIN_PATHS: List[str] = []
+    FLEXMEASURES_PLUGINS: List[str] = []
     FLEXMEASURES_PROFILE_REQUESTS: bool = False
     FLEXMEASURES_DB_BACKUP_PATH: str = "migrations/dumps"
+    FLEXMEASURES_MENU_LOGO_PATH: str = ""
+    FLEXMEASURES_EXTRA_CSS_PATH: str = ""
     FLEXMEASURES_ROOT_VIEW: Union[str, List[Union[str, Tuple[str, List[str]]]]] = []
     FLEXMEASURES_MENU_LISTED_VIEWS: List[Union[str, Tuple[str, List[str]]]] = [
         "dashboard",
-        "analytics",
-        "portfolio",
         "assets",
         "users",
     ]
     FLEXMEASURES_MENU_LISTED_VIEW_ICONS: Dict[str, str] = {}
     FLEXMEASURES_MENU_LISTED_VIEW_TITLES: Dict[str, str] = {}
+    FLEXMEASURES_ASSET_TYPE_GROUPS = {
+        "renewables": ["solar", "wind"],
+        "EVSE": ["one-way_evse", "two-way_evse"],
+    }  # how to group assets by asset types
     FLEXMEASURES_LP_SOLVER: str = "cbc"
+    FLEXMEASURES_JOB_TTL: timedelta = timedelta(days=1)
     FLEXMEASURES_PLANNING_HORIZON: timedelta = timedelta(hours=2 * 24)
     FLEXMEASURES_PLANNING_TTL: timedelta = timedelta(
         days=7
@@ -173,6 +179,7 @@ class TestingConfig(Config):
     LOGGING_LEVEL = logging.INFO
     WTF_CSRF_ENABLED = False  # also necessary for logging in during tests
 
+    SECRET_KEY = "dummy-key-for-testing"
     SECURITY_PASSWORD_SALT = "$2b$19$abcdefghijklmnopqrstuv"
     SQLALCHEMY_DATABASE_URI = (
         "postgresql://flexmeasures_test:flexmeasures_test@localhost/flexmeasures_test"
@@ -191,4 +198,4 @@ class TestingConfig(Config):
 
 
 class DocumentationConfig(Config):
-    pass
+    SECRET_KEY = "dummy-key-for-documentation"
